function example1
% Example 1: Force closure test for grasp of sphere in Chapter on Grasping.

% Written by Jeff Trinkle, March 2013.

% Choose contact locations, models, and friction coefficients
theta1 = atan2(-0.6, -0.8);
theta2 = atan2(-0.6,  0.8);
% Note that for the various parts of example 1, the contact types must be
% changed.
type1 = 'SF';
type2 = 'HF';
mu1 = 0.5;
mu2 = 0.6;
ups = 1;

% Problem parameters
l1 = 3;
l2 = 2;
l3 = 1;
l4 = 2;
l5 = 1;
l6 = 1;
l7 = 0;

% Contact locations
c1 = [cos(theta1); sin(theta1); 0];
c2 = [cos(theta2); sin(theta2); 0];

% Point contact normals into the sphere and o-hat axes out of the page.
R1 = [-cos(theta1)   sin(theta1)   0;
      -sin(theta1)  -cos(theta1)   0;
          0              0         1];
R2 = [-cos(theta2)   sin(theta2)   0;
      -sin(theta2)  -cos(theta2)   0;
          0              0         1];
      
% Here are the definitions of the columns of R1 and R2 in terms of contact
% frame directions.
% n1 = R1(:,1);
% t1 = R1(:,2);
% o1 = R1(:,3);
% 
% n2 = R2(:,1);
% t2 = R2(:,2);
% o2 = R2(:,3);

Rbar1 = [   R1      zeros(3,3);
         zeros(3,3)    R1];
Rbar2 = [   R2      zeros(3,3);
         zeros(3,3)    R2];

S1 = zeros(3,3);
S1(1,[2,3]) = [-c1(3)   c1(2)];
S1(2,[1,3]) = [ c1(3)  -c1(1)];
S1(3,[1,2]) = [-c1(2)   c1(1)];

S2 = zeros(3,3);
S2(1,[2,3]) = [-c2(3)   c2(2)];
S2(2,[1,3]) = [ c2(3)  -c2(1)];
S2(3,[1,2]) = [-c2(2)   c2(1)];

P1 = [eye(3,3)  zeros(3,3);
         S1       eye(3,3)];
                
P2 = [eye(3,3)  zeros(3,3);
         S2       eye(3,3)];
                
tildeG1 = P1 * Rbar1;
tildeG2 = P2 * Rbar2;
tildeG = [tildeG1  tildeG2];     
  
I = eye(6,6);
switch type1
    case('PwoF')
        H1 = I(1,:);
    case('HF')
        H1 = I(1:3, :);
    case('SF')
        H1 = I(1:4, :);
end
switch type2
    case('PwoF')
        H2 = I(1,:);
    case('HF')
        H2 = I(1:3, :);
    case('SF')
        H2 = I(1:4, :);
end
     
H = blkdiag(H1, H2);
G = tildeG * H';
% Also could use  G = [G1  G2];
% G1 = tildeG1 * H1';
% G2 = tildeG2 * H2';

tildeJ1 = [  -l1   -l3;
              l2    l7;
            zeros(3,2);
              1    1]; 
tildeJ1 = [tildeJ1  zeros(6,3)];

sqrt2o2 = sqrt(2) / 2;
J2_32 = sqrt2o2 * (l4 + l5);
tildeJ2 = [  zeros(1,3);
           0     0      l6;
           l4   J2_32    0;
           0  -sqrt2o2  0;
          -1  -sqrt2o2  0
           0     0      1];
tildeJ2 = [zeros(6,2)  tildeJ2];
tildeJ = [tildeJ1;  tildeJ2];
J = H * tildeJ;
% Could also block the Js from each finger.
% J1 = H1 * tildeJ1;
% J2 = H2 * tildeJ2;

% These next few commands are only here to produce the various null space
% bases listed in the chapter.
rnkJ = rank(J);
rnkG = rank(G);
NJ = null(J);
NJt = null(J');
NG = null(G);
NGt = null(G');

% Test force closure for mu2 = 2 and mu1 varying with a soft finger and
% hard finger contact.
type = [3 2];
mu1 = 0.5 : 0.01 : 2;
mu2 = [0.7  0.8  1.3  2.0];
metric = zeros(length(mu1), length(mu2));
for j = 1 : length(mu2)
    for i = 1 : length(mu1)
        mu = [mu1(i)  mu2(j)];
        [frcCl, metricFrFrmCl] = forceClosure(G, J, type, mu);
        if frcCl == 1
            metric(i,j) = metricFrFrmCl;
        end
    end
end
figure;
plot(mu1, metric, 'LineWidth', 2);
title('Closure Metric vs Friction', 'FontSize', 24);
xlabel('\mu_1, friction coefficient at contact 1', 'FontSize', 20)
ylabel('Frictional form closure metric', 'FontSize', 20);
legend([100 220  30  30], '\mu_2 = 0.7', '\mu_2 = 0.8', '\mu_2 = 1.3', '\mu_2 = 2.0');
print(gcf, '-dpdf', 'example1', '-r600');
end
