function example2
% Force closure test for grasp of polygon in Example 2.  Note that
% frictional form closure is achieved only if the contact types are HF and
% if mu1 > 0.75 and mu2 > 0.  If in addition all joints are active, then
% the grasp has force closure.
%
%  Usage: example_2(mu1)

% Create input needed for the force closure test.
%    G     - grasp matrix
%    J     - grasp Jacobian
%    type  - a vector of contact types:
%               1 -->  point without friction
%               2 -->  hard finger contact
%               3 -->  soft finger contact
%    mu    - a vector of friction coefficients

type = [2; 2];    % Define contact types

% Parameters values found from direct measurements of figure
l1 = 2.89;
l2 = 0.75;
l3 = 1.97;
l4 = 0.80;
l5 = 0.80;
l6 = 0.90;  % I forced the ratio l6:l7 = 3:4 so mu1 > 0.75 => frictional form closure
l7 = 1.20;
l8 = 1.35;

% The columns of G  are:  [G1n, G1t, G2n, G2t].
G = [-0.8  -0.6   1   0;
      0.6  -0.8   0   1;
      l6   -l7    0  -l8];
  
% The columns of J' have the same order:  [J'1n, J'1t, J'2n, J'2t].
Jtrn = [ 0.8*l1  0.6*l1   0      0;
        -0.6*l2  0.8*l2   0      0;
           0       0     -l1     0;
           0       0     -l3     0;
           0       0     -l3     l5;
           0       0     -l3+l4  l5;
           0       0     -l3+l4  0];
J = Jtrn';

% Hand is redundant due to the parallelogram struction of the left finger
rnkJ = rank(J);
nullJ = null(J);
qdot_null = [0; 0; 0; -1; 1; -1; 1];  % A null space vector by inspection
err = J*qdot_null;   % Verify qdot_null is in the null space of J.

% Change to PwoF contacts
G13 = G(:, [1 3]);
nullG13 = null(G13');

% Back to full G with HF contacts
nullG = null(G);

% Lock joints 4, 5, 6, and 7.
% This is a different problem, so expect different answers.
J123 = J(:, [1 2 3]);
rnkJ123 = rank(J123);
nullJt123 = null(J123');

% The following plots the metric vs friction at contact 1.  
metric = [];
mu = [0; .8];
mu1 = 0.5 : 0.01 : 3;
for i = 1 : length(mu1)
    mu(1) = mu1(i);
    [frcCl, metricFrFrmCl] = forceClosure(G, J, type, mu);
    metric = [metric   metricFrFrmCl];
end
figure;
plot(mu1, metric);
end
