function example5
% Form closure test for Example 5.
% alpha is the angle of the normal at contact four in radians
% If d > 0, then the grasp has form closure.

% Written by Guiseppe Muscio and Jeff Trinkle
close all

% Example 5, part 1.  Plot form closure metric.
figure
LP1_vals = [];
Ferrari_vals = [];
alpha = 1.04 : .01 : 1.59;
for i = 1 : length(alpha)
    ell = 1;
    Gn = [0    0     1     -cos(alpha(i)) ;
          1    1     0     -sin(alpha(i)) ;
        -ell   0  7*ell/4       0       ];    
    [flag, d] = formClosure(Gn);
    LP1_vals = [LP1_vals     d];
    c(1)='b';
    % This is a cludge for ferrari_canny_metric.  It returns positive
    % distances even when the set convex hull is empty.
    if abs(d) < 1e-3
        d = 0;
    else
        [flag, d] = ferrariCanny(Gn);
    end
    % the 23.086 is used to scale F-C to have the same max as LP1
    Ferrari_vals = [Ferrari_vals    23.086*d];
    c(2)='r';
end
plot(alpha, LP1_vals, c(1), 'linewidth',2);
title('Form Closure Metrics vs Contact Angle \alpha', 'FontSize', 24);
xlabel('\alpha (radians)', 'FontSize', 20);
ylabel('LP1 Metric', 'FontSize', 20);
hold on;
plot (alpha, Ferrari_vals, c(2), 'linewidth',2);
legend('LP1', 'FC');
% print(gcf, '-dpdf', 'example5p1', '-r600');


% Example 5, part 2.  Bilateral constraints.
% The metric, LP1, with bilateral constraints is tested in two ways:
%    Appending Gn with negatives of its original columns
%    Moving some columns of Gn to B and using the annihilator approach
hndl2 = figure;
title('LP1 Metric without Bilateral Constraints', 'FontSize', 24);
xlabel('\alpha (radians)', 'FontSize', 20);
ylabel('LP1 Metric', 'FontSize', 20);
hold on;
% Note that the following figure was not put in the handbook, but it is
% still interesting.
hndl3 = figure;
title('LP1 Metric with Bilateral Constraints', 'FontSize', 24);
xlabel('\alpha (radians)', 'FontSize', 20);
ylabel('LP1 Metric', 'FontSize', 20);
hold on;
alpha = 1.04 : .01 : 1.59;
for j = 1:5
    LP1_vals = [];
    LP2_vals = [];
    B = [];
    for i = 1 : length(alpha)
        ell = 1;
        Gn = [0    0     1     -cos(alpha(i)) ;
              1    1     0     -sin(alpha(i)) ;
            -ell  0  7*ell/4       0           ];
        %
        G1 = [Gn  -Gn(:, 1 : j-1)  ];
        G2 = Gn(:, j : 4);
        B2 = Gn(:, 1 : j-1);
        switch j
            case 1
                c(j) = 'b';
            case 2
                c(j) = 'g';
            case 3
                c(j) = 'k';
            case 4
                c(j) = 'r';
            case 5
                c(j) = 'm';
        end
        [flag, d1] = formClosure(G1);
        [flag, d2] = formClosure(G2, B2);
        LP1_vals = [LP1_vals  d1];
        LP2_vals = [LP2_vals  d2];
    end
    figure(hndl2);
    plot(alpha, LP1_vals, c(j), 'linewidth',2);
    figure(hndl3);
    plot(alpha, LP2_vals, c(j), 'linewidth',2);
end
figure(hndl2);
legend('4','5','6','7','8');
axis([1, 1.6, 0, 8.2]);  % Adjust so I don't lose the top plot when creating figure.
print(gcf, '-dpdf', 'example5p2', '-r600');
figure(hndl3);
legend('4','5','6','7','8');
axis([1, 1.6, 0, 8.2]);  % Adjust so I don't lose the top plot when creating figure.
print(gcf, '-dpdf', 'example5p3', '-r600');


% Example 5, part 3.  Monoticity.
figure;
subplot(2,1,1);
title('LP1 Metric as Number of Contacts Grows', 'FontSize', 24);
xlabel('\alpha (radians)', 'FontSize', 20);
ylabel('LP1 Metric', 'FontSize', 20);
hold on;
subplot(2,1,2); 
title('FC Metric as Number of Contacts Grows', 'FontSize', 24);
xlabel('\alpha (radians)', 'FontSize', 20);
ylabel('FC Metric', 'FontSize', 20);
hold on;
alpha = 1.04 : .01 : 1.59;
e = -1.3;  
for j = 1:6
    LP1_vals = [];
    Ferrari_vals = [];
    for i = 1 : length(alpha)
        ell = 1;
        Gn = [0    0     1     -cos(alpha(i)) ;
              1    1     0     -sin(alpha(i)) ;
            -ell   0  7*ell/4       0           ];
        
        switch j
            case 1
                c(j) = 'b';
                
            case 2
                Gn = [Gn  [-1;
                            0;
                         e*ell]  ];
                c(j) = 'g';
                
            case 3
                Gn = [Gn  [-1          0;
                            0         -1;
                          e*ell    0.5*ell]  ];
                c(j) = 'k';
                
            case 4
                Gn = [Gn  [-1          0          -1;
                            0         -1           0;
                          e*ell    0.5*ell     -7/4*ell]  ];
                c(j) = 'r';
                
            case 5
                Gn = [Gn  [-1         0          -1           1;
                            0        -1           0           0;
                         e*ell     0.5*ell     -7/4*ell     0.6*ell]  ];
                c(j) = 'm';
                
            case 6
                Gn = [Gn  [-1         0          -1           1           0;
                            0        -1           0           0          -1;
                          e*ell    0.5*ell     -7/4*ell     0.6*ell    0.7*ell]  ];
                c(j) = 'c';
                
        end
        [flag, d] = formClosure(Gn);
        LP1_vals = [LP1_vals  d];
        if abs(d) < 1e-3
            d2 = 0;
        else
            [flag, d2] = ferrariCanny(Gn);
        end
        Ferrari_vals = [Ferrari_vals  23.086*d2];
    end
    subplot(2,1,1);
    plot(alpha, LP1_vals, c(j), 'linewidth',2);
    subplot(2,1,2);
    plot(alpha, Ferrari_vals, c(j), 'linewidth',2);
end
subplot(2,1,1);
legend('4','5','6','7','8','9');
subplot(2,1,2);
legend('4','5','6','7','8','9');
print(gcf, '-dpdf', 'example5p4', '-r600');




